library(shiny)

source("charToNum.R")
source("chfun.R")
source("extras.R")
attach(extras)
npoints <- 250

shinyServer(function(input, output,session) {
       
   fx <- reactive({
        from <- charToNum(input$from)[[1]]
        to <- charToNum(input$to)[[1]]
        x <- seq(from,to,length=npoints)
        f<-function(x) {
            eval(parse(text=input$fun))
        }
        xp <- seq(from,to,length=input$kpoints+1)
        if(input$where=="random") xp1 <- runif(input$kpoints,xp[-length(xp)],xp[-1])
        if(input$where=="left") xp1 <- xp[-length(xp)]
        if(input$where=="right") xp1 <- xp[-1]
        if(input$where=="middle") xp1 <- (xp[-1]+xp[-length(xp)])/2 
        list(cbind(x,f(x)),xp,f(xp1),xp1)
   })     
   
   Int <- reactive({
       from <- charToNum(input$from)[[1]]
       to <- charToNum(input$to)[[1]]
       f<-function(x) {
           eval(parse(text=input$fun))
       }
       round(integrate(f,from,to)$value,4)
   })     


   output$text <- renderText({
        from <- charToNum(input$from)[[2]]
        to <- charToNum(input$to)[[2]]
      h <- fx()[[2]][2]-fx()[[2]][1]
      yp <- fx()[[3]]
      Int1 <- round(sum(yp)*h,4)
      fhtml <- chfun(input$fun)

      if(input$where=="left") ftxt <- "f(x<sub>k</sub>)"
      if(input$where=="middle") ftxt <- "f( (x<sub>k+1</sub>+x<sub>k</sub>)/2 )"
      if(input$where=="right") ftxt <- "f(x<sub>k+1</sub>)"
      if(input$where=="random") ftxt <- "f(x<sup>*</sup><sub>k</sub>)"

      line <-    "<table>" 
      line[2] <- paste("<tr><td>",spc10,"&nbsp;</td><td ",al,">",to,"</td><td>&nbsp;</td></tr>")
      line[3] <- paste("<tr><td ",ar,">",spc10,plus2,"&int;</td><td>&nbsp; </td><td ",al,">",plus2,fhtml,"dx = ",Int(),"</td></tr>")
      line[4] <- paste("<tr><td>",spc10,"&nbsp;</td><td ",al,">",from,"</td><td>&nbsp;</td></tr>")
      line[5] <- "<tr><td>&nbsp;</td></tr>"
      line[6] <- paste("<tr><td>",spc10,"&nbsp;</td><td>",input$kpoints,"</td><td>&nbsp;</td></tr>")
      line[7] <- paste("<tr><td>",spc10,plus2,"&sum;</td><td>&nbsp; </td><td>",plus2,ftxt,
                 "(x<sub>k+1</sub>-x<sub>k</sub>)=",Int1,"</td></tr>")
      line[8] <- paste("<tr><td>",spc10,"&nbsp;</td><td>k=0</td><td>&nbsp;</td></tr>")
      line[9] <- "</table>"

      line
   })  
     
    
   output$plot <- renderPlot({
        from <- charToNum(input$from)[[1]]
        to <- charToNum(input$to)[[1]]       
        plot(fx()[[1]],xlab="x",ylab="y",type="l",lwd=3)
        segments(from,0,from,fx()[[1]][1,2],lwd=1)
        segments(to,0,to,fx()[[1]][npoints,2],lwd=1)
        xp <- fx()[[2]]
        yp <- fx()[[3]]
        for(i in 1:input$kpoints) rect(xp[i],0,xp[i+1],yp[i])
        points(fx()[[4]],yp,pch=20,col="red")
   })
           
   output$appinfo <- renderText({ 
      HTML("<br>&nbsp;<br>&nbsp;<br>&nbsp;<br>
      <a target=\"_blank\" href=\"http://academic.uprm.edu/wrolke/myapps1.htm#integral\"><h4>What to do</h4></a><br>
      <h6>Written by: Dr. Wolfgang Rolke<br>Created: 1/22/2015</h6>")
      
  })
  
  output$explainText <- renderText({ 
        "<h5>Explanation: This demo illustrates Riemann Integration</h5>"
        
  })

})

